function  [A_aux,B_aux,C_aux,A_p_aux,B_p_aux,map_als_vec] = ...
                                      ccp_als_hard(Y_1,Y_2,iter,sigma_n,...
                                                   A_0,B_0,C_0,A_p_0,B_p_0)
%-------------------------------------------------------------------------%
% [A_aux,B_aux,C_aux,A_p_aux,B_p_aux,C_p_aux,map_als_vec] = ...
%                                     ccp_als_hard(Y_1,Y_2,iter,sigma_n,...
%                                           ,A_0,B_0,C_0,A_p_0,B_p_0,C_p_0)
%
% ALS algorithm for hard coupled CP decomposition. Factors C is the same in
% both decompositions. Factors A, B, A_p have first row elements equal to 
% 1.
%
% Inputs:
% 
% - Y_1         : first data matrix;
% - Y_2         : second data matrix;
% - iter        : maximum number of iterates;
% - sigma_n     : vector with noise standard deviations;
% - sigma_c     : coupling strenght;
% - A_0,...     : initial factors for the first model;
% - A_p_0,...   : initial factors for the second model.  
%
% Outputs:
% 
% - A,...       : factors for the first model;
% - A_p,...     : factors for the second model;
% - map_als_vec : iterations (Row 1) x MAP objective (Row 2).
%
% External functions:
%
% - kr.m        : Khatri-Rao product;
% - sylvester.m : Sylvester equation solver, necessary if type='alter'. 
%                 Built-in function since Matlab 2014.
%
% List of updates                 -     23/02/2015  -     J. E. Cohen and
%                                                         R. C. Farias 
%                                       Creation of the file
%                                 -     22/07/2015  -     R. C. Farias and
%                                                         J. E. Cohen
%                                       Updated with fast right inverse and
%                                       added options to find C factors
%                                 -     11/02/2016  -     R. C. Farias
%                                       Modification to hard coupling
%                                       setting
%-------------------------------------------------------------------------%

%-------------------------------Parameters--------------------------------%
% Number of iterations for each evaluation of the MAP objective
iter_map  =     10;
% Size of the filter for the relative error decrease
n_diff_map=     3;
% Stopping criteria parameters
stop_diff =     eps;
stop_map  =     eps;
% Maximum number of iterations (/iter_map)
iter      =     ceil(iter/iter_map);
%-------------------------------------------------------------------------%

%-------------------------------Parameters--------------------------------%
% Size of the tensor
dim_1     =     size(Y_1);
dim_2     =     size(Y_2);
R         =     size(A_0,2);
% Index for n-mode matricization
J_1       =     prod(dim_1)./dim_1;
J_2       =     prod(dim_2)./dim_2;

% Three different unfoldings of the tensor
Y_1_mode  =     cell(3);
Y_2_mode  =     cell(3);
% Data tensors unfolding
% First tensor
Y_1_mode{1}     =     reshape(permute(Y_1,[1,2,3]),dim_1(1),J_1(1));  
Y_1_mode{2}     =     reshape(permute(Y_1,[2,1,3]),dim_1(2),J_1(2));
Y_1_mode{3}     =     reshape(permute(Y_1,[3,1,2]),dim_1(3),J_1(3));
% Second tensor
Y_2_mode{1}     =     reshape(permute(Y_2,[1,2,3]),dim_2(1),J_2(1));  
Y_2_mode{2}     =     reshape(permute(Y_2,[2,1,3]),dim_2(2),J_2(2));
Y_2_mode{3}     =     reshape(permute(Y_2,[3,1,2]),dim_2(3),J_2(3));

% Squared norm of the data
snorm_Y_1       =     Y_1_mode{1}(:)'*Y_1_mode{1}(:);
snorm_Y_2       =     Y_2_mode{1}(:)'*Y_2_mode{1}(:);

% Precisions and coupling strength
prec      =     1./(sigma_n.^2);
% Normalization of the MAP objective function function
norm      =     mean(prec);
% Normalized constants in the MAP objective function
prec      =     prec/norm;
%-------------------------------------------------------------------------%

%-----------------------Algorithm initialization--------------------------%
% Filter for the relative error decrease
diff_map_filt   =     1000*ones(1,n_diff_map);
% Storing initial condition on the compressed space
A_aux     =     A_0;
B_aux     =     B_0;
C_aux     =     C_0;
A_p_aux   =     A_p_0;
B_p_aux   =     B_p_0;
% Initial MAP objective value
res_1     =     Y_1_mode{1}-A_aux*kr(C_aux,B_aux)';
err_1     =     res_1(:)'*res_1(:);
res_2     =     Y_2_mode{1}-A_p_aux*kr(C_aux,B_p_aux)';
err_2     =     res_2(:)'*res_2(:);
map_als_old     =     prec(1)*err_1+prec(2)*err_2;
% Vector of squared norms of the error
map_als_vec     =     zeros(1,iter+1);
map_als_vec(1)  =     map_als_old;     
%-------------------------------------------------------------------------%

fprintf('\n Hard coupled CP ALS - CCP ALS \n\n')

%--------------------------Algorithm iterate------------------------------%
t         =     0;
while     (mean(diff_map_filt)>stop_diff &&  t<iter ...
          &&  map_als_old>stop_map   )
    % Iteration counter (counts/iter_map)
    t=    t+1;
    % Coupled ALS (10 iterates)
    for   i           =     1:iter_map
    
    % A factors update
    temp_CB           =     (C_aux'*C_aux).*(B_aux'*B_aux);
    A_aux             =     (Y_1_mode{1}*kr(C_aux,B_aux))/temp_CB;
    
    temp_CB           =     (C_aux'*C_aux).*(B_p_aux'*B_p_aux);
    A_p_aux           =     (Y_2_mode{1}*kr(C_aux,B_p_aux))/temp_CB;
    
    % Normalization
    A_aux       =     A_aux.*repmat(1./A_aux(1,:),dim_1(1),1);
    A_p_aux     =     A_p_aux.*repmat(1./A_p_aux(1,:),dim_2(1),1);
    
    % B factors update
    temp_CA     =     (C_aux'*C_aux).*(A_aux'*A_aux);
    B_aux       =     (Y_1_mode{2}*kr(C_aux,A_aux))/temp_CA;
    
    temp_CA     =     (C_aux'*C_aux).*(A_p_aux'*A_p_aux);
    B_p_aux     =     (Y_2_mode{2}*kr(C_aux,A_p_aux))/temp_CA;
    
    % Normalization
    B_aux       =     B_aux.*repmat(1./B_aux(1,:),dim_1(2),1);
    B_p_aux     =     B_p_aux.*repmat(1./B_p_aux(1,:),dim_2(2),1);
    
    %---------------------------Factor C update---------------------------%
    
    % Intermediate quantities which will be also used to evaluate the MAP
    % objective
    temp_1_BA   =     (B_aux'*B_aux).*(A_aux'*A_aux);
    temp_2_BA   =     (B_p_aux'*B_p_aux).*(A_p_aux'*A_p_aux);
    Y_1_BA=     Y_1_mode{3}*kr(B_aux,A_aux);
    Y_2_BA=     Y_2_mode{3}*kr(B_p_aux,A_p_aux);
    
    % C update
    C_aux =     (Y_1_BA*prec(1)+Y_2_BA*prec(2))/...
                (temp_1_BA*prec(1)+temp_2_BA*prec(2));
    
    %---------------------------------------------------------------------%
    end
    
    % MAP objective value
    err_1       =     snorm_Y_1-2*sum(sum(Y_1_BA.*C_aux))+...
                      sum(sum((C_aux'*C_aux).*temp_1_BA));
    err_2       =     snorm_Y_2-2*sum(sum(Y_2_BA.*C_aux))+...
                      sum(sum((C_aux'*C_aux).*temp_2_BA));
    map_als     =     prec(1)*err_1+prec(2)*err_2;
    
    % Evaluation of the relative error decrease
    diff_map_filt(1:end-1)  =     diff_map_filt(2:end);
    diff_map_filt(end)=     abs((map_als-map_als_old)/map_als_vec(1));
    
    % Storing the last error
    map_als_old        =    map_als;
    map_als_vec(t+1)   =    map_als;
    
    if mod(t*iter_map,100)  ==    0
    % Printing the iterate MAP value and step size
    fprintf('MAP objective value : %g  \tit : %d\n',map_als,t*iter_map)
    end
end
% Deleting zero error trailing at the end
map_als_vec(map_als_vec==0)=[];
map_als_vec=[0 iter_map:iter_map:iter_map*t;map_als_vec];
%-------------------------------------------------------------------------%
end