function CRB  =     crb_ccp_direct_flex(A,B,A_p,B_p,C_p,sigma_n,sigma_c)
%-------------------------------------------------------------------------%
% CRB    =     crb_ccp_direct_flex(A,B,A_p,B_p,C_p,sigma_n,sigma_c)
%
% Hybrid Cramr-Rao bound for the coupled CP decomposition where the 
% factors C are the coupled with a flexible coupling.
%
% The inputs are:
% 
% - A,B         : factors for the first CP model;
% - A_p,B_p     : factors for the second CP model;
% - C_p         : coupled factor;
% - sigma_n     : measurement noise standard deviations;
% - sigma_c     : standard deviation of the coupling noise.
%
% The outputs are:
% 
% - CRB         : Cramr-Rao bound.
%
% List of updates                 -     15/04/2015  -     Rodrigo Cabral
%                                       Creation of the file 
%-------------------------------------------------------------------------%

%-------------------------------Parameters--------------------------------%
% Tensor dimensions
dim       =     [size(A,1) size(B,1) size(C_p,1)];
dim_p     =     [size(A_p,1) size(B_p,1) size(C_p,1)];
R         =     size(A,2);
% Vector of precisions
prec=     1./(sigma_n.^2);
% Coupling intensity
coupl     =     1/(sigma_c^2); 
% Coupling variance
var_c     =     sigma_c^2;  
% Matrices relating the vectorizations of the unfoldings
[J_12,J_13,~]   =     vec_unfold(dim);
% Matrices relating the vectorizations of the unfoldings
[J_12_p,J_13_p,~]     =     vec_unfold(dim_p);
%-------------------------------------------------------------------------%

%---------------------------------FI and CRB------------------------------%
% Information matrices for the hybrid CRB

%-------------------------------------------------------------------------%
% First tensor
% Evaluation of the Khatri-Rao products
ckrb=     kr(C_p,B);
ckra=     kr(C_p,A);   
bkra=     kr(B,A);
% Matrix to evaluate the Fisher information
M   =     [kron(ckrb.',eye(dim(1)));...
          kron(ckra.',eye(dim(2)))*J_12;...
          kron(bkra.',eye(dim(3)))*J_13];
% Direct information matrix
% Uncoupled term
I_d =     (M*(M'));
% Coupling term
I_d_c_AA  =     kron(diag(diag((B'*B))),eye(dim(1)));
I_d_c_BB  =     kron(diag(diag((A'*A))),eye(dim(2)));
R_1       =     cell(R,1);
for r     =     1:R
    R_1{r}=     A(:,r)*(B(:,r)');
end
I_d_c_AB  =     blkdiag(R_1{:});
% Resulting coupling term
I_d_c=    dim(3)*var_c*[I_d_c_AA I_d_c_AB zeros(dim(1)*R,dim(3)*R);...
                            I_d_c_AB' I_d_c_BB zeros(dim(2)*R,dim(3)*R);...
                                               zeros(dim(3)*R,sum(dim)*R)];
% Resulting direct information matrix
I_d =     prec(1)*(I_d+I_d_c);
%-------------------------------------------------------------------------%

%-------------------------------------------------------------------------%
% Second tensor
% Evaluation of the Khatri-Rao products
ckrb_p    =     kr(C_p,B_p);
ckra_p    =     kr(C_p,A_p);   
bkra_p    =     kr(B_p,A_p);
% Matrix to evaluate the Fisher information
M_p       =     [kron(ckrb_p.',eye(dim_p(1)));...
                kron(ckra_p.',eye(dim_p(2)))*J_12_p;...
                kron(bkra_p.',eye(dim_p(3)))*J_13_p];
% Direct information matrix
I_d_p     =     prec(2)*(M_p*(M_p'));
%-------------------------------------------------------------------------%

%-------------------------------------------------------------------------%
% Concatenated information matrix
I_D =     [I_d zeros(sum(dim)*R,sum(dim_p)*R);...
          zeros(sum(dim_p)*R,sum(dim)*R) I_d_p];

% Information matrix for the prior
I_P =     [zeros((dim(1)+dim(2))*R,(sum(dim)+sum(dim_p))*R);...
           zeros(dim(3)*R,(dim(1)+dim(2))*R) coupl*eye(dim(3)*R)...
           zeros(dim(3)*R,(dim_p(1)+dim_p(2))*R)...
           -coupl*eye(dim(3)*R);...
           zeros((dim_p(1)+dim_p(2))*R,(sum(dim)+sum(dim_p))*R);...
           zeros(dim_p(3)*R,(dim(1)+dim(2))*R) ...
           -coupl*eye(dim(3)*R) zeros(dim_p(3)*R,(dim_p(1)+dim_p(2))*R) ...
           coupl*eye(dim(3)*R)];

% Hybrid information matrix
I_H =     I_D+I_P;
%-------------------------------------------------------------------------%
                                  
% Deleting rows and columns of known elements
Ind =     [1 : dim(1) : dim(1)*R...
          dim(1)*R+1 : dim(2) : (dim(1)+dim(2))*R...
          (sum(dim))*R+1 : dim_p(1) : (sum(dim)+dim_p(1))*R...
          (sum(dim)+dim_p(1))*R+1:dim_p(2):(sum(dim)+dim_p(1)+dim_p(2))*R];
                           
I_H(Ind,:)=     [];
I_H(:,Ind)=     [];
% Evaluating the Cramr-Rao bound
CRB     =     inv(I_H);
%-------------------------------------------------------------------------%
end