%--------------------------similar_factors.m------------------------------%
%
% This file evaluates the performance of flexibly coupled CP (CCP ALS) 
% against uncoupled CP (ALS) and hardly coupled CP for coupling with
% similar factors and different coupling intensities. The tensors are
% measured with i.i.d. additive Gaussian noise.
%
% The results are also compared to Cramer-Rao bounds for coupled and 
% uncoupled models.
%
% These simulations are related to Fig. 1 of the paper
% "Exploring multimodal data fusion through joint decompositions with
% flexible couplings".
%
%-------------------------------------------------------------------------%

%--------------Cleaning the workspace and loading functions---------------%
clc;
clear     all;
close     all;
% Add path
addpath('CRB_CCP','CCP_ALS');
%-------------------------------------------------------------------------%

%-----------------------------Model parameters----------------------------%
% 3-way block dimensions
dim       =     [10 10 10];
dim_p     =     [10 10 10];
% Number of components
R         =     3;
% Standard deviations of the measurements noise
sigma_n   =     [0.1 0.001];
% Standard deviation of the coupling
sigma_c   =     [0.5 0.2 0.1 0.05 0.02 0.01 0.005 0.002 0.001 0.0005 ...
                0.0002 0.0001 0.00005 0.00002 0.00001 0.000005];
% Number of iterations for coupled ALS
iter_c    =     1000;
% Number of iterations for uncoupled ALS
iter_u    =     2000;
% Number of realizations for the evaluation of the sum of the MSE
n_real    =     10;
% Number of realizations for the initializations
n_init    =     5;
%-------------------------------------------------------------------------%

%------------------------------Tensor generation--------------------------%
% Fixing the seed to have the same tensor
rng(107)
% Random factors generation
A         =     randn(dim(1),R);
B         =     randn(dim(2),R);
A_p       =     randn(dim_p(1),R);
B_p       =     randn(dim_p(2),R);
C_p       =     randn(dim_p(3),R);

% Normalization of the first rows
A(1,:)    =     ones(1,R);
B(1,:)    =     ones(1,R);
A_p(1,:)  =     ones(1,R);
B_p(1,:)  =     ones(1,R);
%-------------------------------------------------------------------------%

%---------------------------------Coupling model--------------------------%
% Constraint matrix for vectorized factors
H   =     kron(eye(R),eye(dim(3)));
H_p =     kron(eye(R),eye(dim(3)));
%-------------------------------------------------------------------------%

%----------------------------CRB uncoupled CP-----------------------------%
% CRB for the second CP model
[CRB_unc_p FIM_p]     =    crb_cp(A_p,B_p,C_p,sigma_n(2));
%-------------------------------------------------------------------------%

%---------------Simulation for different coupling intensities-------------%
% Progress
N         =     n_real*n_init*length(sigma_c);
n         =     0;

% Allocation for the squared errors
se_A_u    =     zeros(dim(1)*R-R,n_real,length(sigma_c));
se_A_h    =     zeros(dim(1)*R-R,n_real,length(sigma_c));
se_A_c    =     zeros(dim(1)*R-R,n_real,length(sigma_c));
se_A_p_u  =     zeros(dim_p(1)*R-R,n_real,length(sigma_c));
se_A_p_h  =     zeros(dim_p(1)*R-R,n_real,length(sigma_c));
se_A_p_c  =     zeros(dim_p(1)*R-R,n_real,length(sigma_c));
se_B_u    =     zeros(dim(2)*R-R,n_real,length(sigma_c));
se_B_h    =     zeros(dim(2)*R-R,n_real,length(sigma_c));
se_B_c    =     zeros(dim(2)*R-R,n_real,length(sigma_c));
se_B_p_u  =     zeros(dim_p(2)*R-R,n_real,length(sigma_c));
se_B_p_h  =     zeros(dim_p(2)*R-R,n_real,length(sigma_c));
se_B_p_c  =     zeros(dim_p(2)*R-R,n_real,length(sigma_c));
se_C_u    =     zeros(dim(3)*R,n_real,length(sigma_c));
se_C_h    =     zeros(dim(3)*R,n_real,length(sigma_c));
se_C_c    =     zeros(dim(3)*R,n_real,length(sigma_c));
se_C_p_u  =     zeros(dim_p(3)*R,n_real,length(sigma_c));
se_C_p_c  =     zeros(dim_p(3)*R,n_real,length(sigma_c));

% Allocation for the MSE and CRB
mse_A_u               =     zeros(1,length(sigma_c));
mse_B_u               =     zeros(1,length(sigma_c));
mse_C_u               =     zeros(1,length(sigma_c));
mse_A_p_u             =     zeros(1,length(sigma_c));
mse_B_p_u             =     zeros(1,length(sigma_c));
mse_C_p_u             =     zeros(1,length(sigma_c));
mse_A_h               =     zeros(1,length(sigma_c));
mse_B_h               =     zeros(1,length(sigma_c));
mse_C_h               =     zeros(1,length(sigma_c));
mse_A_p_h             =     zeros(1,length(sigma_c));
mse_B_p_h             =     zeros(1,length(sigma_c));
mse_A_c               =     zeros(1,length(sigma_c));
mse_B_c               =     zeros(1,length(sigma_c));
mse_C_c               =     zeros(1,length(sigma_c));
mse_A_p_c             =     zeros(1,length(sigma_c));
mse_B_p_c             =     zeros(1,length(sigma_c));
mse_C_p_c             =     zeros(1,length(sigma_c));
total_CRB_A_u         =     zeros(n_real,length(sigma_c));
total_CRB_B_u         =     zeros(n_real,length(sigma_c));
total_CRB_C_u         =     zeros(n_real,length(sigma_c));
total_CRB_A_p_u       =     zeros(1,length(sigma_c));
total_CRB_B_p_u       =     zeros(1,length(sigma_c));
total_CRB_C_p_u       =     zeros(1,length(sigma_c));
total_CRB_A_c         =     zeros(1,length(sigma_c));
total_CRB_B_c         =     zeros(1,length(sigma_c));
total_CRB_C_c         =     zeros(1,length(sigma_c));
total_CRB_A_p_c       =     zeros(1,length(sigma_c));
total_CRB_B_p_c       =     zeros(1,length(sigma_c));
total_CRB_C_p_c       =     zeros(1,length(sigma_c));
average_CRB_A_u       =     zeros(1,length(sigma_c));
average_CRB_B_u       =     zeros(1,length(sigma_c));
average_CRB_C_u       =     zeros(1,length(sigma_c));
CRB_u                 =     zeros(sum(dim)*R-2*R,sum(dim)*R-2*R,n_real,...
                            length(sigma_c));
                        
% Change the seed to have different noise realizations
%rng('shuffle')
for k     =     1:length(sigma_c)
%-------------------------------------------------------------------------%

%------------------------Different noise realizations---------------------%
for j     =     1:n_real
% Shared factor
C         =     C_p+sigma_c(k)*randn(dim(3),R);
% % Tensor
X         =     reshape(A*transpose(kr(C,B)),dim);
X_p       =     reshape(A_p*transpose(kr(C_p,B_p)),dim_p);

%Noisy measurements
% First noisy tensor
Y         =     X+sigma_n(1)*randn(dim);
% Second noisy tensor
Y_p       =     X_p+sigma_n(2)*randn(dim);

% Different initializations
% Initial cost function values for picking best initialization
obj_n_val_1     =     10^50;
obj_n_val_2     =     10^50;
obj_h_n_val     =     10^50;
map_n_val       =     10^50;
for i     =     1:n_init
%-----------------------------Initialization------------------------------%
A_0       =     randn(size(A));
A_0(1,:)  =     A(1,:);
B_0       =     randn(size(B));
B_0(1,:)  =     B(1,:);
   
A_p_0     =     randn(size(A_p));
A_p_0(1,:)=     A_p(1,:);
B_p_0     =     randn(size(B_p));
B_p_0(1,:)=     B_p(1,:);
 
C_p_0     =     randn(size(C_p));
C_0       =     randn(size(C));
%-------------------------------------------------------------------------%

%-----------------------------Uncoupled CP ALS----------------------------%
% ALS for the first tensor
[A_n_u,B_n_u,C_n_u,obj_val_vec_1] =     als(Y,iter_u,A_0,B_0,C_0);
% ALS for the second tensor
[A_p_n_u,B_p_n_u,C_p_n_u,obj_val_vec_2] =     ...
                                         als(Y_p,iter_u,A_p_0,B_p_0,C_p_0);
[A_p_n_u,B_p_n_u,C_p_n_u]   =    ...
                      amb_correct_1f(A_p_n_u,B_p_n_u,C_p_n_u,C_n_u);
if obj_val_vec_1(end)<obj_n_val_1
obj_n_val_1     =     obj_val_vec_1(end);
A_u             =     A_n_u;
B_u             =     B_n_u;
C_u             =     C_n_u;
end
if obj_val_vec_2(end)<obj_n_val_2
obj_n_val_2     =     obj_val_vec_2(end);
A_p_u           =     A_p_n_u;
B_p_u           =     B_p_n_u;
C_p_u           =     C_p_n_u;
end
%-------------------------------------------------------------------------%

%--------------------------Hard Coupled CP ALS----------------------------%
% Initialization with warm start
A_0       =     A_n_u;
B_0       =     B_n_u;
C_0       =     C_p_n_u;
A_p_0     =     A_p_n_u;
B_p_0     =     B_p_n_u;
% CCP_ALS_HARD
[A_n_h,B_n_h,C_n_h,A_p_n_h,B_p_n_h,obj_h_val_vec]   = ...
                                      ccp_als_hard(Y,Y_p,iter_c,sigma_n,...
                                                  A_0,B_0,C_0,A_p_0,B_p_0);
% Picking best result for different initializations
if obj_h_val_vec(end) <     obj_h_n_val
obj_h_n_val     =     obj_h_val_vec(end);
A_h             =     A_n_h;
B_h             =     B_n_h;
C_h             =     C_n_h;
A_p_h           =     A_p_n_h;
B_p_h           =     B_p_n_h;
end
%-------------------------------------------------------------------------%

%----------------------Flexibly coupled CP ALS----------------------------%
% Initialization with warm start
A_0       =     A_u;
B_0       =     B_u;
C_0       =     C_u;
A_p_0     =     A_p_u;
B_p_0     =     B_p_u;
C_p_0     =     C_p_u;
% CCP_ALS
[A_n_c,B_n_c,C_n_c,A_p_n_c,B_p_n_c,C_p_n_c,map_val_vec] = ...
                 ccp_als(Y,Y_p,iter_c,H,H_p,sigma_n,sigma_c(k),'direct',...
                                            A_0,B_0,C_0,A_p_0,B_p_0,C_p_0);
% Picking best result for different initializations
if map_val_vec(end)<map_n_val
map_n_val       =     map_val_vec(end);
A_c             =     A_n_c;
B_c             =     B_n_c;
C_c             =     C_n_c;
A_p_c           =     A_p_n_c;
B_p_c           =     B_p_n_c;
C_p_c           =     C_p_n_c;
end
%-------------------------------------------------------------------------%
n=n+1;
clc
fprintf('Progress %g %%',(n/N)*100)
end

%--------Correcting the ambiguity and evaluating the squared error--------%
% Uncoupled CP ALS
% Correcting permutation and sign ambiguity for the first tensor
[A_u,B_u,C_u]         =     amb_correct(A_u,B_u,C_u,A,B,C);
% Correcting permutation and sign ambiguity for the second tensor
[A_p_u,B_p_u,C_p_u]   =...
                      amb_correct(A_p_u,B_p_u,C_p_u,A_p,B_p,C_p);                
% Squared errors on the factors
err =     A(2:end,:)-A_u(2:end,:);
se_A_u(:,j,k)         =     (err(:)).^2;
err =     B(2:end,:)-B_u(2:end,:);
se_B_u(:,j,k)         =     (err(:)).^2;
se_C_u(:,j,k)         =     (C(:)-C_u(:)).^2;
err =     A_p(2:end,:)-A_p_u(2:end,:);
se_A_p_u(:,j,k)       =     (err(:)).^2;
err =     B_p(2:end,:)-B_p_u(2:end,:);
se_B_p_u(:,j,k)       =     (err(:)).^2;
se_C_p_u(:,j,k)       =     (C_p(:)-C_p_u(:)).^2;

% Hardly coupled CP ALS
% Correcting permutation and sign ambiguity for the first tensor
[A_h,B_h,C_h]   =     amb_correct(A_h,B_h,C_h,A,B,C);
% Correcting permutation and sign ambiguity for the second tensor
[A_p_h,B_p_h,~] =     amb_correct(A_p_h,B_p_h,C_h,A_p,B_p,C_p);
% Squared errors
err =     A(2:end,:)-A_h(2:end,:);
se_A_h(:,j,k)         =     (err(:)).^2;
err =     B(2:end,:)-B_h(2:end,:);
se_B_h(:,j,k)         =     (err(:)).^2;
se_C_h(:,j,k)         =     (C(:)-C_h(:)).^2;
err =     A_p(2:end,:)-A_p_h(2:end,:);
se_A_p_h(:,j,k)       =     (err(:)).^2;
err =     B_p(2:end,:)-B_p_h(2:end,:);
se_B_p_h(:,j,k)       =     (err(:)).^2;

% Flexibly coupled CP ALS
% Correcting permutation and sign ambiguity for the first tensor
[A_c,B_c,C_c]   =     amb_correct(A_c,B_c,C_c,A,B,C);
% Correcting permutation and sign ambiguity for the second tensor
[A_p_c,B_p_c,C_p_c]   =     amb_correct(A_p_c,B_p_c,C_p_c,A_p,B_p,C_p);
% Squared errors
err =     A(2:end,:)-A_c(2:end,:);
se_A_c(:,j,k)         =     (err(:)).^2;
err =     B(2:end,:)-B_c(2:end,:);
se_B_c(:,j,k)         =     (err(:)).^2;
se_C_c(:,j,k)         =     (C(:)-C_c(:)).^2;
err =     A_p(2:end,:)-A_p_c(2:end,:);
se_A_p_c(:,j,k)       =     (err(:)).^2;
err =     B_p(2:end,:)-B_p_c(2:end,:);
se_B_p_c(:,j,k)       =     (err(:)).^2;
se_C_p_c(:,j,k)       =     (C_p(:)-C_p_c(:)).^2;
%-------------------------------------------------------------------------%

%----------------------------CRB uncoupled CP-----------------------------%
% CRB for the first CP model
[CRB_u(:,:,j,k) FIM]         =    crb_cp(A,B,C,sigma_n(1));
%-------------------------------------------------------------------------%

%------------------------Results for the CRB------------------------------%
% % % Uncoupled results
% % First CP model
% MSE on A
CRB_A_u         =     diag(CRB_u(:,:,j,k));
CRB_A_u         =     CRB_A_u(1:dim(1)*R-R);
% Total MSE on A
total_CRB_A_u(j,k)    =     sum(CRB_A_u);
% MSE on B
CRB_B_u         =     diag(CRB_u(:,:,j,k));
CRB_B_u         =     CRB_B_u(dim(1)*R-R+1:(dim(1)+dim(2))*R-2*R);
% Total MSE on B
total_CRB_B_u(j,k)    =     sum(CRB_B_u);
% MSE on C
CRB_C_u         =     diag(CRB_u(:,:,j,k));
CRB_C_u         =     CRB_C_u((dim(1)+dim(2))*R-2*R+1:end);
% Total MSE on C
total_CRB_C_u(j,k)    =     sum(CRB_C_u);
%-------------------------------------------------------------------------%
end

%----------------------Results for the algorithms-------------------------%
% Mean squared errors on the factors
% Uncoupled CP ALS
mse_A_u(k)      =     sum(mean(se_A_u(:,:,k),2));
mse_B_u(k)      =     sum(mean(se_B_u(:,:,k),2));
mse_C_u(k)      =     sum(mean(se_C_u(:,:,k),2));
mse_A_p_u(k)    =     sum(mean(se_A_p_u(:,:,k),2));
mse_B_p_u(k)    =     sum(mean(se_B_p_u(:,:,k),2));
mse_C_p_u(k)    =     sum(mean(se_C_p_u(:,:,k),2));
% Hard coupled CP ALS
mse_A_h(k)      =     sum(mean(se_A_h(:,:,k),2));
mse_B_h(k)      =     sum(mean(se_B_h(:,:,k),2));
mse_C_h(k)      =     sum(mean(se_C_h(:,:,k),2));
mse_A_p_h(k)    =     sum(mean(se_A_p_h(:,:,k),2));
mse_B_p_h(k)    =     sum(mean(se_B_p_h(:,:,k),2));
% Flexibly coupled CP ALS
mse_A_c(k)      =     sum(mean(se_A_c(:,:,k),2));
mse_B_c(k)      =     sum(mean(se_B_c(:,:,k),2));
mse_C_c(k)      =     sum(mean(se_C_c(:,:,k),2));
mse_A_p_c(k)    =     sum(mean(se_A_p_c(:,:,k),2));
mse_B_p_c(k)    =     sum(mean(se_B_p_c(:,:,k),2));
mse_C_p_c(k)    =     sum(mean(se_C_p_c(:,:,k),2));
%-------------------------------------------------------------------------%

%----------------------------CRB uncoupled CP-----------------------------%
% CRB for the second CP model
[CRB_u_p FIM_p]       =    crb_cp(A_p,B_p,C_p,sigma_n(2));
%-------------------------------------------------------------------------%

%----------------------------CRB Coupled CP-------------------------------%
CRB_c     =    crb_ccp_direct_flex(A,B,A_p,B_p,C_p,sigma_n,sigma_c(k));
%-------------------------------------------------------------------------%

%------------------------Results for the CRB------------------------------%
% % % Uncoupled results
% % Average results for the first CP model
% MSE on A
average_CRB_A_u(k)    =     mean(total_CRB_A_u(:,k));
% MSE on B
average_CRB_B_u(k)    =     mean(total_CRB_B_u(:,k));
% MSE on C
average_CRB_C_u(k)    =     mean(total_CRB_C_u(:,k));

% % Second CP model
% MSE on A
CRB_A_p_u       =     diag(CRB_u_p);
CRB_A_p_u       =     CRB_A_p_u(1:dim_p(1)*R-R);
% Total MSE on A
total_CRB_A_p_u =     sum(CRB_A_p_u);
% MSE on B
CRB_B_p_u       =     diag(CRB_u_p);
CRB_B_p_u       =     CRB_B_p_u(dim_p(1)*R-R+1:(dim_p(1)+dim(2))*R-2*R);
% Total MSE on B
total_CRB_B_p_u =     sum(CRB_B_p_u);
% MSE on C
CRB_C_p_u       =     diag(CRB_u_p);
CRB_C_p_u       =     CRB_C_p_u((dim_p(1)+dim_p(2))*R-2*R+1:end);
% Total MSE on C
total_CRB_C_p_u =     sum(CRB_C_p_u);

% % % Coupled results
% % First CP model
% MSE on A
CRB_A_c         =     diag(CRB_c);
CRB_A_c         =     CRB_A_c(1:(dim(1)*R)-R);
% Total MSE on A
total_CRB_A_c(k)=     sum(CRB_A_c);
% MSE on B
CRB_B_c         =     diag(CRB_c);
CRB_B_c         =     CRB_B_c(dim(1)*R-R+1:(dim(1)+dim(2))*R-2*R);
% Total MSE on B
total_CRB_B_c(k)      =     sum(CRB_B_c);
% MSE on C
CRB_C_c   =     diag(CRB_c);
CRB_C_c   =     CRB_C_c((dim(1)+dim(2))*R-(2*R)+1:sum(dim)*R-2*R);
% Total MSE on C
total_CRB_C_c(k)      =     sum(CRB_C_c);

% % Second CP model
% MSE on A
CRB_A_p_c       =     diag(CRB_c);
CRB_A_p_c       =     CRB_A_p_c(sum(dim)*R-2*R+1:...
                                                sum(dim)*R+dim_p(1)*R-3*R);
% Total MSE on A
total_CRB_A_p_c(k)    =     sum(CRB_A_p_c);
% MSE on B
CRB_B_p_c       =     diag(CRB_c);
CRB_B_p_c       =     CRB_B_p_c(sum(dim)*R+dim_p(1)*R-3*R+1:...
                                     sum(dim)*R+(dim_p(1)+dim_p(2))*R-4*R);
% Total MSE on B
total_CRB_B_p_c(k)    =     sum(CRB_B_p_c);

% MSE on C
CRB_C_p_c       =     diag(CRB_c);
CRB_C_p_c       =     CRB_C_p_c(sum(dim)*R+(dim_p(1)+...
                                                    dim_p(2))*R-4*R+1:end);
% Total MSE on C
total_CRB_C_p_c(k)    =     sum(CRB_C_p_c);
%-------------------------------------------------------------------------%
end
%-------------------------------------------------------------------------%

%---------------------------Results Fig. 1--------------------------------%
loglog(1./sigma_c,average_CRB_C_u,'k--',1./sigma_c,total_CRB_C_c,'b',...
       1./sigma_c,total_CRB_C_p_u*ones(1,length(sigma_c)),'k',...
       1./sigma_c,mse_C_c,'r',1./sigma_c,mse_C_h,'m')
xlabel('Coupling intensity 1/\sigma_c')
ylabel('Total MSE')
legend('Average CRB on C - Uncoupled','CRB on C - Flexible coupling',...
       'CRB on C_p - Uncoupled','MSE on C - Flexible coupling',...
       'MSE on C - Hard coupling')
%-------------------------------------------------------------------------%