%-------------------------------compression.m-----------------------------%
%
% This file evaluates the performance of flexibly coupled CP (CCP ALS) 
% with joint compression against coupled CP ALS in the uncompressed domain 
% and uncoupled CP ALS (jointly compressed, independently compressed and
% uncompressed).The tensors are measured with i.i.d. additive Gaussian 
% noise.
%
% These simulations are related to Fig. 3 of the paper
% "Exploring multimodal data fusion through joint decompositions with
% flexible couplings".
%
%-------------------------------------------------------------------------%

%--------------Cleaning the workspace and loading functions---------------%
clc;
clear     all;
close     all;
% Add path
addpath('CCCP_ALS');
%-------------------------------------------------------------------------%

%-----------------------------Model parameters----------------------------%
% 3-way block uncompressed dimensions
dim       =     [100 100 100];
dim_p     =     [100 100 100];
% 3-way block compressed dimensions
dim_c     =     [5 5 5];
dim_p_c   =     [5 5 5];
% Number of components
R         =     5;
% Standard deviation of the measurements noise in the first tensor
sigma_n_1 =     0.0018;
% Signal to noise ratio for the second tensor
SNR_p     =     [20 18 16 14 12 10 8 6 4];
% Standard deviation of the coupling
sigma_c   =     10^(-3);
% Number of iterates for the ALS algorithms
itermax   =     1000;
% Number of randomly drawn tensors
n_trial   =     50;
% Number of realizations for the evaluation of the sum of the MSE
n_init    =     10;
%-------------------------------------------------------------------------%

%---------------Simulation for different noise levels---------------------%
% Noise parameters for the second tesnor
SNR       =     10*log10(R/sigma_n_1^2/dim(1)/dim(2));
noise_2   =     sqrt(R/dim_p(1)/dim_p(2)./10.^(SNR_p/10));

%--------------------------Different initializations----------------------%
% Allocation of the results
Stock     =     zeros(n_init,length(SNR_p),7);
% Progress
N         =     n_init*n_trial*length(SNR_p);
n_prog    =     0;
for trial =     1:n_trial
indice    =     0;
for sig_2 =     noise_2
    indice=     indice+1;
%------------------------------Tensor generation--------------------------%
% CP model and noisy measurements (additive Gaussian noise)
A   =     randn(dim(1),R);
B   =     randn(dim(2),R);
C   =     randn(dim(3),R);
A_p =     randn(dim_p(1),R);
B_p =     randn(dim_p(2),R);

% Normalization
A   =     A.*repmat(1./sqrt(sum(A.^2)),dim(1),1);
B   =     B.*repmat(1./sqrt(sum(B.^2)),dim(2),1);
%C   =     C.*repmat(1./sqrt(sum(C.^2)),dim(3),1);
A_p =     A_p.*repmat(1./sqrt(sum(A_p.^2)),dim_p(1),1);
B_p =     B_p.*repmat(1./sqrt(sum(B_p.^2)),dim_p(2),1);

% Relating C and C_p
C_p =     C+sigma_c*randn(dim(3),R);

% Noisy measurements
% First noisy tensor
Y_true    =     MultProd(eyeT(R,R),A,B,C);
Y         =     Y_true+sigma_n_1*randn(dim);
Y_m       =     reshape(permute(Y,[3,1,2]),dim(3),dim(2)*dim(1));
% Second noisy tensor
Y_p_true  =     MultProd(eyeT(R,R),A_p,B_p,C_p);
Y_p       =     Y_p_true+sig_2*randn(dim_p);
Y_p_m     =     reshape(permute(Y_p,[3,1,2]),dim_p(3),dim_p(2)*dim_p(1));
%-------------------------------------------------------------------------%

%----------------------- Tucker3 Coupled Compression ---------------------%
% Independent tesnor compression using RSVD
[G_ind,U,V,W_ind]           =     hosvd(Y,dim_c,'random');
[G_p_ind,U_p,V_p,W_p_ind]   =     hosvd(Y_p,dim_p_c,'random');

% Coupled tensor compression
[W,~,~]   =     rsvd([Y_m/sigma_n_1,Y_p_m/sig_2],dim_c(3),1);
W_p       =     W;
G         =     MultProd(Y,U',V',W');
G_p       =     MultProd(Y_p,U_p',V_p',W');
%-------------------------------------------------------------------------%

%------------------------Different noise realizations---------------------%
for n     =     1:n_init

%-----------------------------Initialization------------------------------%
% Uncompressed factors
A_0 =     randn(dim(1),R);
B_0 =     randn(dim(2),R);
C_0 =     randn(dim(3),R);
C_0 =     C_0.*repmat(1./sqrt(sum(C_0.^2)),dim(3),1);
A_0 =     A_0.*repmat(1./sqrt(sum(A_0.^2)),dim(1),1);
B_0 =     B_0.*repmat(1./sqrt(sum(B_0.^2)),dim(2),1);

A_p_0     =     randn(dim_p(1),R);
B_p_0     =     randn(dim_p(2),R);
C_p_0     =     randn(dim_p(3),R);
C_p_0     =     C_p_0.*repmat(1./sqrt(sum(C_p_0.^2)),dim_p(3),1);
A_p_0     =     A_p_0.*repmat(1./sqrt(sum(A_p_0.^2)),dim_p(1),1);
B_p_0     =     B_p_0.*repmat(1./sqrt(sum(B_p_0.^2)),dim_p(2),1);

% Compressed factors
Cc_0      =     W'*C_0;
Ac_0      =     U'*A_0;
Bc_0      =     V'*B_0;
Cc_p_0    =     W_p'*C_p_0;
Ac_p_0    =     U_p'*A_p_0;
Bc_p_0    =     V_p'*B_p_0;
%-------------------------------------------------------------------------%

%----------------------------Uncoupled CP ALS-----------------------------%
% Uncompressed ALS

[A_als,B_als,C_als,err_als_vec]         =     als(Y,itermax,A_0,B_0,C_0);
[A_p_als,B_p_als,C_p_als,err_als_vec_p] =     als(Y_p,itermax,...
                                                        A_p_0,B_p_0,C_p_0);



% Jointly compressed ALS
[Ac_als,Bc_als,Cc_als,errc_als_vec]     =     als(G,itermax,...
                                                           Ac_0,Bc_0,Cc_0);
[Ac_p_als,Bc_p_als,Cc_p_als,errc_als_vec_p]   =     als(G_p,itermax,...
                                                     Ac_p_0,Bc_p_0,Cc_p_0);


% Independently compressed ALS
[Aci_als,Bci_als,Cci_als,errci_als_vec] =     als(G_ind,itermax,...
                                                           Ac_0,Bc_0,Cc_0);
[Aci_p_als,Bci_p_als,Cci_p_als,errci_als_vec_p]   =     als(G_p_ind,...
                                             itermax,Ac_p_0,Bc_p_0,Cc_p_0);
%-------------------------------------------------------------------------%

%--------------Correcting the ambiguity on coupled factor------------------%
[A_als,B_als,C_als]         =     amb_correct_1f(A_als,B_als,C_als,C_p_als);

[Ac_als,Bc_als,Cc_als]      =     amb_correct_1f(Ac_als,Bc_als,Cc_als,...
                                                                Cc_p_als);
                                                            
%---------------------------Coupled CP ALS--------------------------------%
% Uncompressed ALS
tic
[Aj_als,Bj_als,Cj_als, Aj_p_als,Bj_p_als,Cj_p_als,errj_als_vec]   =     ...
        ccp_als(Y,Y_p,itermax/10,1,1,[sigma_n_1,sig_2],sigma_c,'direct',...
                                A_als,B_als,C_als,A_p_als,B_p_als,C_p_als);
t2=toc;

% Compressed ALS
tic
[Ajc_als,Bjc_als,Cjc_als,Ajc_p_als,Bjc_p_als,Cjc_p_als,errjc_als_vec] = ...
        ccp_als(G,G_p,itermax/10,1,1,[sigma_n_1,sig_2],sigma_c,'direct',...
                          Ac_als,Bc_als,Cc_als,Ac_p_als,Bc_p_als,Cc_p_als);
t1=toc;

% Decompression
Cc_als    =     W*Cc_als;
Bc_als    =     V*Bc_als;
Ac_als    =     U*Ac_als;

Cc_p_als  =     W_p*Cc_p_als;
Bc_p_als  =     V_p*Bc_p_als;
Ac_p_als  =     U_p*Ac_p_als;

Cci_als   =     W_ind*Cci_als;
Bci_als   =     V*Bci_als;
Aci_als   =     U*Aci_als;

Cci_p_als =     W_p_ind*Cci_p_als;
Bci_p_als =     V_p*Bci_p_als;
Aci_p_als =     U_p*Aci_p_als;

Cjc_als   =     W*Cjc_als;
Bjc_als   =     V*Bjc_als;
Ajc_als   =     U*Ajc_als;

Cjc_p_als =     W_p*Cjc_p_als;
Bjc_p_als =     V_p*Bjc_p_als;
Ajc_p_als =     U_p*Ajc_p_als;
%-------------------------------------------------------------------------%

%--------Correcting the ambiguity and evaluating the squared error--------%
[A_als,B_als,C_als]         =     amb_correct(A_als,B_als,C_als,A,B,C);
[A_p_als,B_p_als,C_p_als]   =     amb_correct(A_p_als,B_p_als,C_p_als,...
                                                              A_p,B_p,C_p);

[Aj_als,Bj_als,Cj_als]=     amb_correct(Aj_als,Bj_als,Cj_als,A,B,C);
[Aj_p_als,Bj_p_als,Cj_p_als]   = amb_correct(Aj_p_als,Bj_p_als,Cj_p_als,...
                                                              A_p,B_p,C_p);

[Ac_als,Bc_als,Cc_als]      =     amb_correct(Ac_als,Bc_als,Cc_als,A,B,C);
[Ac_p_als,Bc_p_als,Cc_p_als]=    amb_correct(Ac_p_als,Bc_p_als,Cc_p_als,...
                                                              A_p,B_p,C_p);

[Aci_als,Bci_als,Cci_als]   =   amb_correct(Aci_als,Bci_als,Cci_als,A,B,C);
[Aci_p_als,Bci_p_als,Cci_p_als]= amb_correct(Aci_p_als,Bci_p_als,...
                                                    Cci_p_als,A_p,B_p,C_p);

[Ajc_als,Bjc_als,Cjc_als]   =   amb_correct(Ajc_als,Bjc_als,...
                                                            Cjc_als,A,B,C);
[Ajc_p_als,Bjc_p_als,Cjc_p_als]= amb_correct(Ajc_p_als,Bjc_p_als,...
                                                    Cjc_p_als,A_p,B_p,C_p);

Stock(n,indice,:) = [sum(sum((C_p-C_p_als).^2)),...
                sum(sum((C_p-Cj_p_als)).^2),sum(sum((C_p-Cc_p_als)).^2),...
          sum(sum((C_p-Cjc_p_als)).^2),sum(sum((C_p-Cci_p_als)).^2),t1,t2];
%-------------------------------------------------------------------------%
      
% Progress
n_prog    =     n_prog+1;
clc
fprintf('Progress %g %%',(n_prog/N)*100)
end
%-------------------------------------------------------------------------%

end
%-------------------------------------------------------------------------%

% Results
Stock_piled       = min(Stock);
Results(:,:,trial)  = Stock_piled(1,:,:);

end
%-------------------------------------------------------------------------%

%---------------------------Results Fig. 3--------------------------------%
Results_centered  = mean(Results,3);
semilogy(20:-2:4,Results_centered(:,1:5))
legend('ALS','JALS','JCompressed ALS','JCompressed JALS','Compressed ALS')
%-------------------------------------------------------------------------%