function [A_aux,B_aux,C_aux,A_p_aux,B_p_aux,C_p_aux,map_val_vec] =      ...
                          ccp_mu(Y_1,Y_2,iter,beta_n,phi_n,beta_c,phi_c,...
                                             A_0,B_0,C_0,A_p_0,B_p_0,C_p_0)
%-------------------------------------------------------------------------%
% [A_aux,B_aux,C_aux,A_p_aux,B_p_aux,C_p_aux,map_val_vec] =      ...
%                  ccp_mu(Y_1,Y_2,iter_ccp_mu,beta_n,phi_n,beta_c,phi_c,...
%                                            A_0,B_0,C_0,A_p_0,B_p_0,C_p_0)
% Multiplicative update algorithm for coupled nonnegative 3-way tensor 
% decomposition in the presence of Tweedie's noise.
%
% The inputs are:
% 
% - Y_1,Y_2     : data blocks;
% - beta_n      : shape parameter of the Tweedie's noise;
% - beta_c      : shape parameter of the Tweedie's coupling;
% - phi_n       : dispersion parameter of the noise;
% - phi_c       : dispersion parameter of the coupling;
% - iter_ccp_mu : maximum number of iterates;
% - A_0,...     : initial parameters for the factors.
%
% The outputs are:
% 
% - A_aux,...   : factors for the first CP model;
% - A_p_aux,... : factors for the second CP model;
% - map_val_vec : vector of cost function values.
%
% List of updates                 -     03/04/2015  -     Rodrigo Cabral
%                                       Creation of the file 
%-------------------------------------------------------------------------%

%-------------------------------Parameters--------------------------------%
% Size of the filter for the relative error decrease
n_diff_map      =     3;
% Stopping criteria parameters
stop_diff =     eps;
%-------------------------------------------------------------------------%

%-----------------------Algorithm initialization--------------------------%
% Size of the tensors
dim_1     =     size(Y_1);
dim_2     =     size(Y_2);
% Indexes for n-mode matricizations
J_1       =     prod(dim_1)./dim_1;
J_2       =     prod(dim_2)./dim_2;
% Three different unfoldings of the tensor
Y_1_mode  =     cell(3);
Y_2_mode  =     cell(3);
% Data tensors unfoldings
Y_1_mode{1}     =     reshape(permute(Y_1,[1,2,3]),dim_1(1),J_1(1));  
Y_1_mode{2}     =     reshape(permute(Y_1,[2,1,3]),dim_1(2),J_1(2));
Y_1_mode{3}     =     reshape(permute(Y_1,[3,1,2]),dim_1(3),J_1(3));
Y_2_mode{1}     =     reshape(permute(Y_2,[1,2,3]),dim_2(1),J_2(1));  
Y_2_mode{2}     =     reshape(permute(Y_2,[2,1,3]),dim_2(2),J_2(2));
Y_2_mode{3}     =     reshape(permute(Y_2,[3,1,2]),dim_2(3),J_2(3));
% Storing auxiliary variables
A_aux     =     abs(A_0);
B_aux     =     abs(B_0);
C_aux     =     abs(C_0);
A_p_aux   =     abs(A_p_0);
B_p_aux   =     abs(B_p_0);
C_p_aux   =     abs(C_p_0);
% Filter for the relative error decrease
diff_map_filt   =     1000*ones(1,n_diff_map);
% CP model in the first unfolding
% Initial CP models in the first unfolding
X_1             =     A_aux*(kr(C_aux,B_aux)');
X_2             =     A_p_aux*(kr(C_p_aux,B_p_aux)');
% Initial squared norm of the error
map_val_old    =     map_eval(Y_1_mode{1},X_1,Y_2_mode{2},X_2,beta_n,...
                     phi_n,C_aux,C_p_aux,beta_c,phi_c);
% Vector of squared norms of the error
map_val_vec    =     zeros(1,iter);
%-------------------------------------------------------------------------%

fprintf('\nCoupled nonnegative CP decomposition\n')
fprintf('using MU\n\n')

%--------------------------Algorithm iterate------------------------------%
t         =     0;
while     (mean(diff_map_filt)>stop_diff &&  t<iter   )
    % Iteration counter
    t=t+1;
    
    % Multiplicative update for factor A
    % CP model unfolding in the first mode
    CkrB  =     kr(C_aux,B_aux);
    X_1   =     A_aux*(CkrB');
    % Powers of the CP model
    X_1_b =     X_1.^(-beta_n(1));
    X_1_m_b     =     X_1_b.*X_1;
    % A update
    A_aux =     A_aux.*(((Y_1_mode{1}.*X_1_b)*CkrB)./((X_1_m_b)*CkrB));
    % Increasing values too close to zero
    A_aux(A_aux<eps)  =    eps;
    % Normalization
    C_aux =     C_aux.*repmat(sum(A_aux),dim_1(3),1);
    A_aux =     A_aux.*repmat(1./sum(A_aux),dim_1(1),1);
    
    % Multiplicative update for factor A_p
    % CP model unfolding in the first mode
    CkrB_p=     kr(C_p_aux,B_p_aux);
    X_1_p =     A_p_aux*(CkrB_p');
    % Powers of the CP model
    X_1_b_p     =     X_1_p.^(-beta_n(2));
    X_1_m_b_p   =     X_1_b_p.*X_1_p;
    % A update
    A_p_aux     =     A_p_aux.*(((Y_2_mode{1}.*X_1_b_p)*CkrB_p)./...
                      ((X_1_m_b_p)*CkrB_p));
    % Increasing values too close to zero
    A_p_aux(A_p_aux<eps)    =    eps;
    % Normalization
    C_p_aux     =     C_p_aux.*repmat(sum(A_p_aux),dim_2(3),1);
    A_p_aux     =     A_p_aux.*repmat(1./sum(A_p_aux),dim_2(1),1);
    
    % Multiplicative update for factor B
    % CP model unfolding in the first mode
    CkrA  =     kr(C_aux,A_aux);
    X_2   =     B_aux*(CkrA');
    % Powers of the CP model
    X_2_b =     X_2.^(-beta_n(1));
    X_2_m_b     =     X_2_b.*X_2;
    % A update
    B_aux =     B_aux.*(((Y_1_mode{2}.*X_2_b)*CkrA)./((X_2_m_b)*CkrA));
    % Increasing values too close to zero
    B_aux(B_aux<eps)  =    eps;
    % Normalization
    C_aux =     C_aux.*repmat(sum(B_aux),dim_1(3),1);
    B_aux =     B_aux.*repmat(1./sum(B_aux),dim_1(2),1);
    
    % Multiplicative update for factor B
    % CP model unfolding in the first mode
    CkrA_p      =     kr(C_p_aux,A_p_aux);
    X_2_p =     B_p_aux*(CkrA_p');
    % Powers of the CP model
    X_2_b_p     =     X_2_p.^(-beta_n(2));
    X_2_m_b_p   =     X_2_b_p.*X_2_p;
    % A update
    B_p_aux     =     B_p_aux.*(((Y_2_mode{2}.*X_2_b_p)*CkrA_p)./...
                      ((X_2_m_b_p)*CkrA_p));
    % Increasing values too close to zero
    B_p_aux(B_p_aux<eps)    =    eps;
    % Normalization
    C_p_aux     =     C_p_aux.*repmat(sum(B_p_aux),dim_2(3),1);
    B_p_aux     =     B_p_aux.*repmat(1./sum(B_p_aux),dim_2(2),1);
    
    % Multiplicative update for factor C
    % Storing previous C factor
    C_old =     C_aux;
    % CP model unfolding in the first mode
    BkrA  =     kr(B_aux,A_aux);
    X_3   =     C_aux*(BkrA');
    % Powers of the CP model
    X_3_b =     X_3.^(-beta_n(1));
    X_3_m_b     =     X_3_b.*X_3;
    % A update
    C_aux =     C_aux.*(( ...
                      (1/phi_n(1))*(Y_1_mode{3}.*X_3_b)*BkrA + ...
                      (1/(phi_c*(beta_c-1)))*(C_aux.^(1-beta_c)))...
                      ./...
                      ((1/phi_n(1))*(X_3_m_b)*BkrA + ...
                      (beta_c/2)*(1./C_aux) + ...
                      (1/(phi_c*(beta_c-1)))*(C_p_aux.^(1-beta_c))));
    % Increasing values too close to zero
    C_aux(C_aux<eps)  =    eps;
                      
    % Multiplicative update for factor C_p
    % CP model unfolding in the first mode
    BkrA_p=     kr(B_p_aux,A_p_aux);
    X_3_p =     C_p_aux*(BkrA_p');
    % Powers of the CP model
    X_3_b_p     =     X_3_p.^(-beta_n(2));
    X_3_m_b_p   =     X_3_b_p.*X_3_p;
    % A update
    C_p_aux     =     C_p_aux.*( ...
                          (((1/phi_n(2))*(Y_2_mode{3}.*X_3_b_p)*BkrA_p)+...
                          ((1/phi_c)*C_aux.*(C_p_aux.^(-beta_c))))...
                          ./...
                          ((1/phi_n(2))*(X_3_m_b_p)*BkrA_p + ...
                          (1/phi_c)*C_p_aux.^(1-beta_c)));
    % Increasing values too close to zero
    C_p_aux(C_p_aux<eps)  =    eps;
    
    % Evaluation of the beta divergence
    map_val     =     map_eval(Y_1_mode{3},X_3,Y_2_mode{3},X_3_p,beta_n,...
                      phi_n,C_aux,C_p_aux,beta_c,phi_c);
    
    % Evaluation of the relative error decrease
    diff_map_filt(1:end-1)  =     diff_map_filt(2:end);
    diff_map_filt(end)      =     abs((map_val-map_val_old)/map_val);
    
    % Storing the last error
    map_val_old       =     map_val;
    map_val_vec(t+1)  =     map_val;
    
     if (100 * round(double(t)/100) == t)
    % Printing the iterate objective value
    fprintf('Objective value : %g  \tit : %d\n',map_val,t)
    end
end
% Deleting zero error trailing at the end
map_val_vec(map_val_vec==0)=[];
%-------------------------------------------------------------------------%
end