function [A_aux,B_aux,C_aux,beta_div_vec]   = ...
                                                mu(Y,beta,iter,A_0,B_0,C_0)
%-------------------------------------------------------------------------%
% [A_aux,B_aux,C_aux,err_mu_vec]   = ...
%                                            mu(Y,beta,iter_mu,A_0,B_0,C_0)
% Multiplicative update algorithm for nonnegative 3-way tensor decomposi-
% tion in the presence of Tweedie's noise.
%
% The inputs are:
% 
% - Y           : data block;
% - beta        : shape parameter of the Tweedie's noise;
% - iter_mu     : maximum number of iterates;
% - A_0,B_0,C_0 : initial parameters for the factors.
%
% The outputs are:
% 
% - A_aux       : first factor;
% - B_aux       : second factor;
% - C_aux       : third factor;
% - err_als_vec : vector of errors.
%
% List of updates                 -     02/04/2015  -     Rodrigo Cabral
%                                       Creation of the file 
%-------------------------------------------------------------------------%

%-------------------------------Parameters--------------------------------%
% Size of the filter for the relative error decrease
n_diff_beta     =     3;
% Stopping criteria parameters
stop_diff =     eps;
stop_beta =     eps;
%-------------------------------------------------------------------------%

%-----------------------Algorithm initialization--------------------------%
% Size of the tensor
dim       =     size(Y);
% Index for n-mode matricization
J         =     prod(dim)./dim;
% Three different unfoldings of the tensor
Y_mode    =     cell(3);
% Data tensor unfolding
Y_mode{1} =     reshape(permute(Y,[1,2,3]),dim(1),J(1));  
Y_mode{2} =     reshape(permute(Y,[2,1,3]),dim(2),J(2));
Y_mode{3} =     reshape(permute(Y,[3,1,2]),dim(3),J(3));
% Storing auxiliary variables
A_aux     =     abs(A_0);
B_aux     =     abs(B_0);
C_aux     =     abs(C_0);
% Filter for the relative error decrease
diff_beta_filt  =     1000*ones(1,n_diff_beta);
% CP model in the first unfolding
% Initial CP model in the first unfolding
X_1             =     A_aux*(kr(C_aux,B_aux)');
% Initial squared norm of the error
beta_div_old    =     beta_div_eval(Y_mode{1},X_1,beta);
% Vector of squared norms of the error
beta_div_vec    =     zeros(1,iter);
%-------------------------------------------------------------------------%

fprintf('\nNonnegative CP decomposition\n')
fprintf('using MU\n\n')

%--------------------------Algorithm iterate------------------------------%
t         =     0;
while     (mean(diff_beta_filt)>stop_diff &&  t<iter ...
          &&  beta_div_old>stop_beta   )
    % Iteration counter
    t=t+1;
    
    % Multiplicative update for factor A
    % CP model unfolding in the first mode
    CkrB  =     kr(C_aux,B_aux);
    X_1   =     A_aux*(CkrB');
    % Powers of the CP model
    X_1_b =     X_1.^(-beta);
    X_1_m_b     =     X_1_b.*X_1;
    % A update
    A_aux =     A_aux.*(((Y_mode{1}.*X_1_b)*CkrB)./((X_1_m_b)*CkrB));
    % Increasing values too close to zero
    A_aux(A_aux<eps)  =    eps;
    % Normalization
    C_aux =     C_aux.*repmat(sum(A_aux),dim(3),1);
    A_aux =     A_aux.*repmat(1./sum(A_aux),dim(1),1);
    
    % Multiplicative update for factor B
    % CP model unfolding in the first mode
    CkrA  =     kr(C_aux,A_aux);
    X_2   =     B_aux*(CkrA');
    % Powers of the CP model
    X_2_b =     X_2.^(-beta);
    X_2_m_b     =     X_2_b.*X_2;
    % A update
    B_aux =     B_aux.*(((Y_mode{2}.*X_2_b)*CkrA)./((X_2_m_b)*CkrA));
    % Increasing values too close to zero
    B_aux(B_aux<eps)  =    eps;
    % Normalization
    C_aux =     C_aux.*repmat(sum(B_aux),dim(3),1);
    B_aux =     B_aux.*repmat(1./sum(B_aux),dim(2),1);
    
    % Multiplicative update for factor C
    % CP model unfolding in the first mode
    BkrA  =     kr(B_aux,A_aux);
    X_3   =     C_aux*(BkrA');
    % Powers of the CP model
    X_3_b =     X_3.^(-beta);
    X_3_m_b     =     X_3_b.*X_3;
    % A update
    C_aux =     C_aux.*(((Y_mode{3}.*X_3_b)*BkrA)./((X_3_m_b)*BkrA));
    % Increasing values too close to zero
    C_aux(C_aux<eps)  =    eps;
    
    % Evaluation of the beta divergence
    beta_div    =     beta_div_eval(Y_mode{3},X_3,beta);
    
    % Evaluation of the relative error decrease
    diff_beta_filt(1:end-1) =     diff_beta_filt(2:end);
    diff_beta_filt(end)     =     abs((beta_div-beta_div_old)/beta_div);
    
    % Storing the last error
    beta_div_old      =     beta_div;
    beta_div_vec(t+1) =     beta_div;
    
    if (100 * round(double(t)/100) == t)
    % Printing the iterate objective value
    fprintf('Objective value : %g  \tit : %d\n',beta_div,t)
    end
end
% Deleting zero error trailing at the end
beta_div_vec(beta_div_vec==0)=[];
%-------------------------------------------------------------------------%
end